﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using System;
using System.Collections.Generic;
using System.Linq;
using HEAL.Attic;

namespace HeuristicLab.Problems.DataAnalysis.Implementation {
  [Item("NamedIntervals", "Represents variables with their interval ranges.")]
  [StorableType("230B4E4B-41E5-4D33-9BC3-E2DAADDCA5AE")]
  public class NamedIntervals : Item {
    Dictionary<string, Interval> variableIntervals = new Dictionary<string, Interval>();
    public Dictionary<string, Interval> VariableIntervals => variableIntervals;

    [Storable(Name = "StorableIntervalInformation")]
    private KeyValuePair<string, double[]>[] StorableIntervalInformation {
      get {
        var l = new List<KeyValuePair<string, double[]>>();
        foreach (var varInt in variableIntervals)

          l.Add(new KeyValuePair<string, double[]>(varInt.Key,
            new double[] { varInt.Value.LowerBound, varInt.Value.UpperBound }));
        return l.ToArray();
      }

      set {
        foreach (var varInt in value)
          variableIntervals.Add(varInt.Key, new Interval(varInt.Value[0], varInt.Value[1]));
      }
    }

    public NamedIntervals() : base() { }
    [StorableConstructor]
    protected NamedIntervals(StorableConstructorFlag _) : base(_) { }

    protected NamedIntervals(NamedIntervals original, Cloner cloner) : base(original, cloner) {
      foreach (var keyValuePair in original.VariableIntervals) {
        variableIntervals.Add(keyValuePair.Key, keyValuePair.Value);
      }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new NamedIntervals(this, cloner);
    }

    public bool ReadOnly { get; }

    public bool InsertMany(IEnumerable<KeyValuePair<string, Interval>> entries) {
      if (entries == null) throw new ArgumentNullException("The given dataset is null.");
      if (entries.Count() != 0) {
        foreach (var entry in entries) {
          if (variableIntervals.ContainsKey(entry.Key))
            variableIntervals[entry.Key] = entry.Value;
          else
            variableIntervals.Add(entry.Key, entry.Value);
        }
      }
      return true;
    }

    public bool Add(string key, Interval value) {
      variableIntervals[key] = value;
      return true;
    }

  }
}

